import { userInfoResponseSchema } from '@/schemas';
import { useUserStore } from '@/stores';
import { createRouter, createWebHistory, type RouteRecordRaw } from 'vue-router';
import axiosInstance from '@/api';
import { ElMessage } from 'element-plus';
import { errorMessage } from '@/utils';
import { AxiosError } from 'axios';
import { RoutePermission } from './permissions';

export * from './permissions';

declare module 'vue-router' {
  // noinspection JSUnusedGlobalSymbols
  interface RouteMeta {
    permission: RoutePermission;
  }
}
const routes: RouteRecordRaw[] = [
  {
    path: '/',
    component: () => import('@/views/MainPage.vue'),
    meta: {
      permission: RoutePermission.MAIN_PAGE
    }
  },
  {
    path: '/user',
    component: () => import('@/views/UserPage.vue'),
    meta: {
      permission: RoutePermission.USER_PAGE
    }
  },
  {
    path: '/user/:id',
    component: () => import('@/views/UserPage.vue')
  },
  {
    path: '/club',
    component: () => import('@/views/ClubPage.vue'),
    meta: {
      permission: RoutePermission.CLUB_PAGE
    }
  },
  {
    path: '/2048',
    component: () => import('@/views/Game2048Page.vue')
  },
  {
    path: '/:brokenPath',
    redirect: '/'
  }
];
const router = createRouter({
  history: createWebHistory(),
  routes: routes
});
router.beforeEach(async (to) => {
  console.log(to);
  const userStore = useUserStore();
  console.log(userStore.isInitialized);
  if (!userStore.isInitialized) {
    try {
      const userInfoResponse = userInfoResponseSchema.parse(
        (await axiosInstance.get('/api/user/info')).data
      );
      if (userInfoResponse.type === 'error') {
        ElMessage.error(
          errorMessage('获取用户信息失败', userInfoResponse.code, userInfoResponse.msg)
        );
        return false;
      }
      // 判断是否已登录.
      if (userStore.token !== null) {
        userStore.updateUserInfo(userInfoResponse);
      }
      userStore.permissions.push(...userInfoResponse.data.auth.permissions);
    } catch (e) {
      if (e instanceof AxiosError) {
        ElMessage.error(errorMessage('获取用户信息失败', e.code, e.message));
      }
      return false;
    } finally {
      userStore.isInitialized = true;
    }
  }
  if (
    to.meta.permission === undefined ||
    userStore.permissions.find((fullPermission) => fullPermission.id === to.meta.permission)
  ) {
    return true;
  } else {
    return '/';
  }
});
export default router;
